@@ -67,7 +67,7 @@
 
 size_t writen2(int fd, const void *buf, size_t count)
 {
-  const char *ptr = (char*)buf;
+  const char *ptr = reinterpret_cast<const char*>(buf);
   const char *eptr = ptr + count;
 
   ssize_t res;
@@ -110,26 +110,30 @@ size_t readn2(int fd, void* buffer, size_t len)
   return len;
 }
 
-size_t readn2WithTimeout(int fd, void* buffer, size_t len, int idleTimeout, int totalTimeout)
+size_t readn2WithTimeout(int fd, void* buffer, size_t len, const struct timeval& idleTimeout, const struct timeval& totalTimeout, bool allowIncomplete)
 {
   size_t pos = 0;
-  time_t start = 0;
-  int remainingTime = totalTimeout;
-  if (totalTimeout) {
-    start = time(NULL);
+  struct timeval start{0,0};
+  struct timeval remainingTime = totalTimeout;
+  if (totalTimeout.tv_sec != 0 || totalTimeout.tv_usec != 0) {
+    gettimeofday(&start, nullptr);
   }
 
   do {
     ssize_t got = read(fd, (char *)buffer + pos, len - pos);
     if (got > 0) {
       pos += (size_t) got;
+      if (allowIncomplete) {
+        break;
+      }
     }
     else if (got == 0) {
       throw runtime_error("EOF while reading message");
     }
     else {
       if (errno == EAGAIN) {
-        int res = waitForData(fd, (totalTimeout == 0 || idleTimeout <= remainingTime) ? idleTimeout : remainingTime);
+        struct timeval w = ((totalTimeout.tv_sec == 0 && totalTimeout.tv_usec == 0) || idleTimeout <= remainingTime) ? idleTimeout : remainingTime;
+        int res = waitForData(fd, w.tv_sec, w.tv_usec);
         if (res > 0) {
           /* there is data available */
         }
@@ -144,14 +148,15 @@ size_t readn2WithTimeout(int fd, void* buffer, size_t len, int idleTimeout, int
       }
     }
 
-    if (totalTimeout) {
-      time_t now = time(NULL);
-      int elapsed = now - start;
-      if (elapsed >= remainingTime) {
+    if (totalTimeout.tv_sec != 0 || totalTimeout.tv_usec != 0) {
+      struct timeval now;
+      gettimeofday(&now, nullptr);
+      struct timeval elapsed = now - start;
+      if (remainingTime < elapsed) {
         throw runtime_error("Timeout while reading data");
       }
       start = now;
-      remainingTime -= elapsed;
+      remainingTime = remainingTime - elapsed;
     }
   }
   while (pos < len);
@@ -159,11 +164,11 @@ size_t readn2WithTimeout(int fd, void* buffer, size_t len, int idleTimeout, int
   return len;
 }
 
-size_t writen2WithTimeout(int fd, const void * buffer, size_t len, int timeout)
+size_t writen2WithTimeout(int fd, const void * buffer, size_t len, const struct timeval& timeout)
 {
   size_t pos = 0;
   do {
-    ssize_t written = write(fd, (char *)buffer + pos, len - pos);
+    ssize_t written = write(fd, reinterpret_cast<const char *>(buffer) + pos, len - pos);
 
     if (written > 0) {
       pos += (size_t) written;
@@ -172,7 +177,7 @@ size_t writen2WithTimeout(int fd, const void * buffer, size_t len, int timeout)
       throw runtime_error("EOF while writing message");
     else {
       if (errno == EAGAIN) {
-        int res = waitForRWData(fd, false, timeout, 0);
+        int res = waitForRWData(fd, false, timeout.tv_sec, timeout.tv_usec);
         if (res > 0) {
           /* there is room available */
         }
@@ -222,7 +227,7 @@ uint32_t getLong(const unsigned char* p)
 
 uint32_t getLong(const char* p)
 {
-  return getLong((unsigned char *)p);
+  return getLong(reinterpret_cast<const unsigned char *>(p));
 }
 
 static bool ciEqual(const string& a, const string& b)
@@ -466,9 +471,9 @@ void stripLine(string &line)
 string urlEncode(const string &text)
 {
   string ret;
-  for(string::const_iterator i=text.begin();i!=text.end();++i)
-    if(*i==' ')ret.append("%20");
-    else ret.append(1,*i);
+  for(char i : text)
+    if(i==' ')ret.append("%20");
+    else ret.append(1,i);
   return ret;
 }
 
@@ -565,13 +570,33 @@ string makeHexDump(const string& str)
   string ret;
   ret.reserve((int)(str.size()*2.2));
 
-  for(string::size_type n=0;n<str.size();++n) {
-    snprintf(tmp, sizeof(tmp), "%02x ", (unsigned char)str[n]);
+  for(char n : str) {
+    snprintf(tmp, sizeof(tmp), "%02x ", (unsigned char)n);
     ret+=tmp;
   }
   return ret;
 }
 
+string makeBytesFromHex(const string &in) {
+  if (in.size() % 2 != 0) {
+    throw std::range_error("odd number of bytes in hex string");
+  }
+  string ret;
+  ret.reserve(in.size() / 2);
+
+  unsigned int num;
+  for (size_t i = 0; i < in.size(); i += 2) {
+    const auto numStr = in.substr(i, 2);
+    num = 0;
+    if (sscanf(numStr.c_str(), "%02x", &num) != 1) {
+      throw std::range_error("Invalid value while parsing the hex string '" + in + "'");
+    }
+    ret.push_back(static_cast<uint8_t>(num));
+  }
+
+  return ret;
+}
+
 void normalizeTV(struct timeval& tv)
 {
   if(tv.tv_usec > 1000000) {
@@ -688,7 +713,7 @@ int makeIPv6sockaddr(const std::string& addr, struct sockaddr_in6* ret)
     hints.ai_flags = AI_NUMERICHOST;
 
     // getaddrinfo has anomalous return codes, anything nonzero is an error, positive or negative
-    if (getaddrinfo(ourAddr.c_str(), 0, &hints, &res) != 0) {
+    if (getaddrinfo(ourAddr.c_str(), nullptr, &hints, &res) != 0) {
       return -1;
     }
 
@@ -724,7 +749,7 @@ int makeIPv4sockaddr(const std::string& str, struct sockaddr_in* ret)
   if(!*(str.c_str() + pos + 1)) // trailing :
     return -1;
 
-  char *eptr = (char*)str.c_str() + str.size();
+  char *eptr = const_cast<char*>(str.c_str()) + str.size();
   int port = strtol(str.c_str() + pos + 1, &eptr, 10);
   if (port < 0 || port > 65535)
     return -1;
@@ -793,7 +818,7 @@ Regex::Regex(const string &expr)
 // Note that cmsgbuf should be aligned the same as a struct cmsghdr
 void addCMsgSrcAddr(struct msghdr* msgh, cmsgbuf_aligned* cmsgbuf, const ComboAddress* source, int itfIndex)
 {
-  struct cmsghdr *cmsg = NULL;
+  struct cmsghdr *cmsg = nullptr;
 
   if(source->sin4.sin_family == AF_INET6) {
     struct in6_pktinfo *pkt;
@@ -1109,8 +1134,8 @@ string getMACAddress(const ComboAddress& ca)
       if (sscanf(parts[3].c_str(), "%02x:%02x:%02x:%02x:%02x:%02x", tmp, tmp+1, tmp+2, tmp+3, tmp+4, tmp+5) != 6) {
         return ret;
       }
-      for(int i = 0 ; i< 6 ; ++i)
-        ret.append(1, (char)tmp[i]);
+      for(unsigned int i : tmp)
+        ret.append(1, (char)i);
       return ret;
     }
   }
@@ -1122,25 +1147,103 @@ uint64_t udpErrorStats(const std::string& str)
 {
 #ifdef __linux__
   ifstream ifs("/proc/net/snmp");
-  if(!ifs)
+  if (!ifs) {
     return 0;
+  }
+
+  string line;
+  while (getline(ifs, line)) {
+    if (boost::starts_with(line, "Udp: ") && isdigit(line.at(5))) {
+      vector<string> parts;
+      stringtok(parts, line, " \n\t\r");
+
+      if (parts.size() < 7) {
+        break;
+      }
+
+      if (str == "udp-rcvbuf-errors") {
+        return std::stoull(parts.at(5));
+      }
+      else if (str == "udp-sndbuf-errors") {
+        return std::stoull(parts.at(6));
+      }
+      else if (str == "udp-noport-errors") {
+        return std::stoull(parts.at(2));
+      }
+      else if (str == "udp-in-errors") {
+        return std::stoull(parts.at(3));
+      }
+      else if (parts.size() >= 8 && str == "udp-in-csum-errors") {
+        return std::stoull(parts.at(7));
+      }
+      else {
+        return 0;
+      }
+    }
+  }
+#endif
+  return 0;
+}
+
+uint64_t udp6ErrorStats(const std::string& str)
+{
+#ifdef __linux__
+  const std::map<std::string, std::string> keys = {
+    { "udp6-in-errors", "Udp6InErrors" },
+    { "udp6-recvbuf-errors", "Udp6RcvbufErrors" },
+    { "udp6-sndbuf-errors", "Udp6SndbufErrors" },
+    { "udp6-noport-errors", "Udp6NoPorts" },
+    { "udp6-in-csum-errors", "Udp6InCsumErrors" }
+  };
+
+  auto key = keys.find(str);
+  if (key == keys.end()) {
+    return 0;
+  }
+
+  ifstream ifs("/proc/net/snmp6");
+  if (!ifs) {
+    return 0;
+  }
+
+  std::string line;
+  while (getline(ifs, line)) {
+    if (!boost::starts_with(line, key->second)) {
+      continue;
+    }
+
+    std::vector<std::string> parts;
+    stringtok(parts, line, " \n\t\r");
+
+    if (parts.size() != 2) {
+      return 0;
+    }
+
+    return std::stoull(parts.at(1));
+  }
+#endif
+  return 0;
+}
+
+uint64_t tcpErrorStats(const std::string& str)
+{
+#ifdef __linux__
+  ifstream ifs("/proc/net/netstat");
+  if (!ifs) {
+    return 0;
+  }
+
   string line;
   vector<string> parts;
-  while(getline(ifs,line)) {
-    if(boost::starts_with(line, "Udp: ") && isdigit(line[5])) {
+  while (getline(ifs,line)) {
+    if (line.size() > 9 && boost::starts_with(line, "TcpExt: ") && isdigit(line.at(8))) {
       stringtok(parts, line, " \n\t\r");
-      if(parts.size() < 7)
-	break;
-      if(str=="udp-rcvbuf-errors")
-	return std::stoull(parts[5]);
-      else if(str=="udp-sndbuf-errors")
-	return std::stoull(parts[6]);
-      else if(str=="udp-noport-errors")
-	return std::stoull(parts[2]);
-      else if(str=="udp-in-errors")
-	return std::stoull(parts[3]);
-      else
-	return 0;
+
+      if (parts.size() < 21) {
+        break;
+      }
+
+      return std::stoull(parts.at(20));
     }
   }
 #endif
@@ -1255,6 +1358,9 @@ uint64_t getOpenFileDescriptors(const std::string&)
   closedir(dirhdl);
   return ret;
 
+#elif defined(__OpenBSD__)
+  // FreeBSD also has this in libopenbsd, but I don't know if that's available always
+  return getdtablecount();
 #else
   return 0;
 #endif
@@ -1270,7 +1376,9 @@ uint64_t getRealMemoryUsage(const std::string&)
   uint64_t size, resident, shared, text, lib, data;
   ifs >> size >> resident >> shared >> text >> lib >> data;
 
-  return data * getpagesize();
+  // We used to use "data" here, but it proves unreliable and even is marked "broken"
+  // in https://www.kernel.org/doc/html/latest/filesystems/proc.html 
+  return resident * getpagesize();
 #else
   struct rusage ru;
   if (getrusage(RUSAGE_SELF, &ru) != 0)
@@ -1344,7 +1452,7 @@ uid_t strToUID(const string &str)
   const char * cstr = str.c_str();
   struct passwd * pwd = getpwnam(cstr);
 
-  if (pwd == NULL) {
+  if (pwd == nullptr) {
     long long val;
 
     try {
@@ -1373,7 +1481,7 @@ gid_t strToGID(const string &str)
   const char * cstr = str.c_str();
   struct group * grp = getgrnam(cstr);
 
-  if (grp == NULL) {
+  if (grp == nullptr) {
     long long val;
 
     try {
@@ -1583,3 +1691,66 @@ std::string getCarbonHostName()
 
   return hostname;
 }
+
+std::string makeLuaString(const std::string& in)
+{
+  ostringstream str;
+
+  str<<'"';
+
+  char item[5];
+  for (unsigned char n : in) {
+    if (islower(n) || isupper(n)) {
+      item[0] = n;
+      item[1] = 0;
+    }
+    else {
+      snprintf(item, sizeof(item), "\\%03d", n);
+    }
+    str << item;
+  }
+
+  str<<'"';
+
+  return str.str();
+}
+
+size_t parseSVCBValueList(const std::string &in, vector<std::string> &val) {
+  std::string parsed;
+  auto ret = parseRFC1035CharString(in, parsed);
+  parseSVCBValueListFromParsedRFC1035CharString(parsed, val);
+  return ret;
+};
+
+#ifdef HAVE_CRYPTO_MEMCMP
+#include <openssl/crypto.h>
+#else /* HAVE_CRYPTO_MEMCMP */
+#ifdef HAVE_SODIUM_MEMCMP
+#include <sodium.h>
+#endif /* HAVE_SODIUM_MEMCMP */
+#endif /* HAVE_CRYPTO_MEMCMP */
+
+bool constantTimeStringEquals(const std::string& a, const std::string& b)
+{
+  if (a.size() != b.size()) {
+    return false;
+  }
+  const size_t size = a.size();
+#ifdef HAVE_CRYPTO_MEMCMP
+  return CRYPTO_memcmp(a.c_str(), b.c_str(), size) == 0;
+#else /* HAVE_CRYPTO_MEMCMP */
+#ifdef HAVE_SODIUM_MEMCMP
+  return sodium_memcmp(a.c_str(), b.c_str(), size) == 0;
+#else /* HAVE_SODIUM_MEMCMP */
+  const volatile unsigned char *_a = (const volatile unsigned char *) a.c_str();
+  const volatile unsigned char *_b = (const volatile unsigned char *) b.c_str();
+  unsigned char res = 0;
+
+  for (size_t idx = 0; idx < size; idx++) {
+    res |= _a[idx] ^ _b[idx];
+  }
+
+  return res == 0;
+#endif /* !HAVE_SODIUM_MEMCMP */
+#endif /* !HAVE_CRYPTO_MEMCMP */
+}
