@@ -97,7 +97,10 @@ static Json getServerDetail() {
     { "daemon_type", productTypeApiType() },
     { "version", getPDNSVersion() },
     { "config_url", "/api/v1/servers/localhost/config{/config_setting}" },
-    { "zones_url", "/api/v1/servers/localhost/zones{/zone}" }
+    { "zones_url", "/api/v1/servers/localhost/zones{/zone}" },
+#ifndef RECURSOR
+    { "autoprimaries_url", "/api/v1/servers/localhost/autoprimaries{/autoprimary}" }
+#endif
   };
 }
 
@@ -117,6 +120,20 @@ void apiDiscovery(HttpRequest* req, HttpResponse* resp) {
   resp->setJsonBody(doc);
 }
 
+void apiDiscoveryV1(HttpRequest* req, HttpResponse* resp) {
+  if(req->method != "GET")
+    throw HttpMethodNotAllowedException();
+
+  Json version1 = Json::object {
+    { "server_url", "/api/v1/servers{/server}" },
+    { "api_features", Json::array {} }
+  };
+  Json doc = Json::array { version1 };
+
+  resp->setJsonBody(doc);
+
+}
+
 void apiServer(HttpRequest* req, HttpResponse* resp) {
   if(req->method != "GET")
     throw HttpMethodNotAllowedException();
